<?php
/**
 * iF.SVNAdmin
 * Copyright (c) 2010 by Manuel Freiholz
 * http://www.insanefactory.com/
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; version 2
 * of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.
 */
class IF_SVNClientC
{
  private $m_svnExe = NULL;

  /**
   * Constructor.
   */
  public function __construct()
  {
  }

  /**
   * Initializes the object with the absolute path to the
   * command line executable.
   * 
   * @param string $svnExecutable The command line client
   */
  public function init( $svnExecutable )
  {
    $this->m_svnExe = $svnExecutable;
    return true;
  }

  /**
   * Gets an array with files and folders of the given repository+path.
   * Note: This function currently only works with a local file system repository path.
   *
   * The type in the returning array can be:
   *   0 = FOLDER
   *   1 = FILE
   *
   * @param string $repository Absolute path to the repository (no trailing slash)
   * @param string $relPath Relative path inside the repository (without leading slash)
   * @return array<string,string> with FILENAME=>TYPE names or FALSE on error.
   */
  public function ls( $repository, $relPath )
  {
    // Due to platform compatibility it is important that the absolute
    // repository path ($repository) begins with a slash (/).
    // YES, also on windows systems: /C:/mypath/tomy/repo
    // If the repository path ended with a slash, it have to be removed.

    if (!file_exists($repository))
      return false;

    // Get the position of the first occurance of a slash.
    $slashCheck = strpos( $repository, "/" );
    $len = strlen( $repository );

    // Remove ending slash.
    if( $slashCheck !== FALSE && $slashCheck == $len-1 )
      $repository = substr( $repository, 0, -1 );

    // Add leading slash.
    if( $slashCheck === FALSE || $slashCheck > 0 )
      $repository = "/".$repository;


    // If a $relPath is given, it must not begin with a slash.
    // Remove the leading slash.
    if( $relPath == NULL )
      $relPath = "";
    elseif( $relPath == "" )
      ;
    else
    {
      $slashCheck = strpos( $relPath, "/" );
      if( $slashCheck !== FALSE && $slashCheck == 0 )
        $relPath = substr( $relPath, 1 );
    }


    // Build command.
    // Example(win): svn.exe ls "file:///C:/repos/repo1/path/in/repo"
    // Example(unix): svn.exe ls "file:///repos/repo1/path/in/repo"
    $cmd = $this->m_svnExe;// \"".$repository."/".$relPath."\"";
    $cmd.= " ls ";
    $cmd.= escapeshellarg( "file://".$repository."/".$relPath );

    $exitCode = 0;
    $cmdOutput = array();
    exec( $cmd, $cmdOutput, $exitCode );

    if( $exitCode != 0 )
    { // Something went wrong... Maybe the $relPath points to a file?
      return false;
    }

    $retFileList = array();
    $lineCount = count( $cmdOutput );
    for( $i = 0; $i < $lineCount; $i++ )
    {
      // Is it a file or folder?
      $pos = strpos( $cmdOutput[$i], "/" );
      if( $pos !== false )
      { // directory
        $dirName = substr( $cmdOutput[$i], 0, -1 ); // Remove the ending slash.
        $retFileList[$dirName] = 0;
      }
      else
      { // file
        $retFileList[$cmdOutput[$i]] = 1;
      }
    }
    return $retFileList;
  }
}
?>