
/**
 * Inserts alternate row colors to the given table.
 * @param eTable The table Element
 * @return void
 */
function alternateRowColors( eTable ){
  var table = eTable;
  var rows = table.getElementsByTagName( "TR" );

  var i = 0;
  for( i = 0; i < rows.length; i++ ){
    // Skip THEAD and TFOOT element rows.
    var parentTagName = rows[i].parentNode.tagName;
    if( parentTagName == "TFOOT" || parentTagName == "THEAD" ){
      continue;
    }
    // Change className of all other rows.
    if( i % 2 == 0 ){
      rows[i].className = "trbase";
    }
    else{
      rows[i].className = "tralt";
    }
  }
}

/**
 * Searches for all "datatables" and alternates the row colors of them.
 * @return void
 */
function highlightDatatables(){
  var tables = document.getElementsByTagName("table");
  var ti = 0;
  for( ti = 0; ti < tables.length; ti++ ){
    if( tables[ti].className.indexOf("datatable") != -1 ){
      alternateRowColors( tables[ti] );
    }
  }
}

/**
 * Filter existing datatables
 * @param tableid
 * @param colidx
 * @param text
 * @return void
 */
function filterDataTable( tableid, colidx, text )
{
  var strSearchText = new String(text);
  if( strSearchText.length == 1 ){
    if( strSearchText == "*" ){
      strSearchText = "\\*";
    }
  }

  var table = document.getElementById(tableid);
  var rows = table.getElementsByTagName("TR");

  var ri = 0;
  for( ri = 0; ri < rows.length; ri ++ ){
    var cells = rows[ri].getElementsByTagName("TD");

    // Continue search, when the requested colidx exists.
    if( cells.length >= (colidx+1) ){
      var content = cells[colidx].data;
      if( cells[colidx].hasChildNodes() ){
        var last = cells[colidx];
        while( last.hasChildNodes() ){
          last = last.firstChild;
        }
        content = last.data;
      }

      // At this place the contentn is filled with the cell's content.
      var str = new String(content);
      if( str.match( strSearchText ) ){
        // Show row.
        rows[ri].style.display = "";
      }
      else{
        // Hide row.
        rows[ri].style.display = "none";
      }

    }
  }
}

function deletionPrompt(question) {
  var agree = confirm(question);
  if (agree) {
    return true;
  }
  return false;
}

function openRoleListWindow()
{
  var url = "rolelist.php";
  var attrs = "width=600,height=400,status=no,scrollbars=yes,resizable=yes";
  window.open(url, 'rolelist', attrs);
}

function selectAll(srcObj, targetCheckBoxName)
{
  var checked_status = srcObj.checked;
  $("input[name='"+targetCheckBoxName+"'][type='checkbox']").each(function(){
    this.checked = checked_status;
  });
}

/**
 * Checks whether the test has been passed.
 * @return bool
 */
function isTestPassed(xml){
  if ($('passed', xml).text() == "1"){
    return true;
  }
  return false;
}

/**
 * Executes an test.
 * @param testSection
 * @param requestVars
 * @param resultContainer
 */
function testSettings(testSection, requestVars, resultContainer)
{
  var url = 'settings.php?dotest=1&dotestsec='+testSection;

  // Hide the result element.
  $(resultContainer).hide('fast');

  $.post(url, requestVars,
    function(xml){
      // Check whether we receive a valid response.
      if ($('message', xml).text() == "")
        alert("Invalid response data:\n"+xml);
      else {
        // Has the test been passed?
        var passed = isTestPassed(xml);

        // Set the result message.
        $(resultContainer).html($('message', xml).text());

        // Show the result element.
        $(resultContainer).show('fast');

        $(resultContainer).removeClass();
        if (passed){
          $(resultContainer).addClass("okmsg");
        }
        else {
          $(resultContainer).addClass("errormsg");
        }
      }
    }
  );
}

function updateSettingsSelection()
{
  // UserView
  var speed = "slow";
  if ($("#UserViewProviderType").val() == "off")
  {
    $("#tbl_userfile").hide(speed);
    $("#tbl_ldapconnection").hide(speed);
    $("#tbl_ldapuser").hide(speed);
    $("#UserEditProviderType").val("off");
    $("#UserEditProviderType").attr("disabled", "disabled");
  }
  else if($("#UserViewProviderType").val() == "passwd")
  {
    $("#tbl_ldapconnection").hide(speed);
    $("#tbl_ldapuser").hide(speed);
    $("#tbl_userfile").show(speed);
    $("#UserEditProviderType").removeAttr("disabled");
  }
  else if($("#UserViewProviderType").val() == "ldap")
  {
    $("#tbl_userfile").hide(speed);
    $("#tbl_ldapconnection").show(speed);
    $("#tbl_ldapuser").show(speed);
    $("#UserEditProviderType").val("off");
    $("#UserEditProviderType").attr("disabled", "disabled");
  }

  // Group view
  if ($("#GroupViewProviderType").val() == "off")
  {
    if ($("#UserViewProviderType").val() != "ldap"){
      $("#tbl_ldapconnection").hide(speed);
    }
    $("#tbl_ldapgroup").hide(speed);
    $("#GroupEditProviderType").val("off");
    $("#GroupEditProviderType").attr("disabled", "disabled");
  }
  else if ($("#GroupViewProviderType").val() == "svnauthfile")
  {
    if ($("#UserViewProviderType").val() != "ldap"){
      $("#tbl_ldapconnection").hide(speed);
    }
    $("#tbl_ldapgroup").hide(speed);
    $("#GroupEditProviderType").removeAttr("disabled");
  }
  else if ($("#GroupViewProviderType").val() == "ldap")
  {
    if ($("#UserViewProviderType").val() == "ldap")
    {
      $("#tbl_ldapconnection").show(speed);
      $("#tbl_ldapgroup").show(speed);
      $("#GroupEditProviderType").val("off");
      $("#GroupEditProviderType").attr("disabled", "disabled");
    }
    else
    {
      $("#GroupViewProviderType").val("off");
      $("#tbl_ldapgroup").hide(speed);
      $("#GroupEditProviderType").val("off");
      $("#GroupEditProviderType").attr("disabled", "disabled");
      alert("The users must be fetched from LDAP, if you want to use the groups from your LDAP server.");
    }
  }

  // Repository view
  if ($("#RepositoryViewProviderType").val() == "off")
  {
    $("#tbl_subversion").hide(speed);
    $("#RepositoryEditProviderType").val("off");
    $("#RepositoryEditProviderType").attr("disabled", "disabled");
  }
  else if ($("#RepositoryViewProviderType").val() == "svnclient")
  {
    $("#tbl_subversion").show(speed);
    $("#RepositoryEditProviderType").removeAttr("disabled");
  }
}